/* BOREXINO Reconstruction program
 *
 * Author: Davide D'Angelo <davide.dangelo@lngs.infn.it>
 *
 * $Id: bx_muon_event.cc,v 1.1 2004/04/27 09:41:09 ddangelo Exp $
 *
 * Implementation of bx_muon_event.hh
 *
 */

#include "bx_muon_event.hh"
#include "bx_event_disk_format.h"
#include "messenger.hh"

const unsigned long bx_muon_edge::tdc_chip_mask      = 0x03;// << 24;
const unsigned long bx_muon_edge::tdc_channel_mask   = 0x1f;// << 18;
const unsigned long bx_muon_edge::slope_mask         = 0x01;// << 16;
const unsigned long bx_muon_edge::overflow_mask      = 0x01;// << 17;
const unsigned long bx_muon_edge::raw_time_mask      = 0xff      ; 
const unsigned long bx_muon_edge::event_number_mask  = 0x01;// << 23;
const unsigned long bx_muon_edge::last_datum_mask    = 0x01;// << 30;
const unsigned long bx_muon_edge::invalid_datum_mask = 0x01;// << 31;
 

bx_muon_edge::bx_muon_edge (const char *disk_edge) {
  
  muon_edge_disk_format *edge = (muon_edge_disk_format *)disk_edge;
  
  u4_tdc_word   = edge->tdc_word;
}

const unsigned short bx_muon_raw_hit::led_reference_channel = 255;
const unsigned short bx_muon_raw_hit::yellow_pulse_channel = 254;
const unsigned short bx_muon_raw_hit::clock_pulse_channel = 253;
const unsigned short bx_muon_raw_hit::omt_pulse_channel = 252;

bx_muon_raw_hit::bx_muon_raw_hit (const bx_muon_edge& first_edge, const bx_muon_edge& second_edge, unsigned char board) : lead_edge(first_edge), 
															  trail_edge(second_edge), 
															  u1_tdc_board(board) {
}

// Performs preliminary checks on the edge couple. Returns 0 on error.
bx_muon_raw_hit::check_t bx_muon_raw_hit::check_validity () const {
    //    msg << "event " << dispatch;
  if (lead_edge.get_slope () == trail_edge.get_slope ()
      || lead_edge.get_tdc_chip () != trail_edge.get_tdc_chip ()
      || lead_edge.get_tdc_channel () != trail_edge.get_tdc_channel ())
    return missing_edge;

  if (lead_edge.get_time () < trail_edge.get_time ())
    // print a warning;
    return inverted;

  if (get_muon_channel() > 255) {
    bx_message msg(bx_message::error, "bx_muon_raw_hit::check_validity() ");
    msg << "Invalid muon channel. mch " << get_muon_channel() << "; brd " << u1_tdc_board << "; chip " << int(lead_edge.get_tdc_chip()) << " ; chann " << int(lead_edge.get_tdc_channel()) << dispatch;
  }
  return fine;
}
  
bx_muon_raw_event::bx_muon_raw_event (const char *disk_event) {

  muon_header_disk_format *head = (muon_header_disk_format *)disk_event;
  if (head->nwords == 0) return;

  u4_nedges = head->nwords - sizeof (muon_header_disk_format) / sizeof (unsigned long);
  u4_trgid = head->trgid;
  b_error_flag = bool (head->error_flag);

  raw_hits.clear ();
  unsigned char chip = 0;
  for (unsigned long i = 0; i < u4_nedges; i++) {
    int board = chip / constants::muon.tdc.chips_per_board;
    bx_muon_edge el = bx_muon_edge(disk_event + sizeof (muon_header_disk_format) + i * sizeof (muon_edge_disk_format));
    if (el.is_last ()) chip++;
    if (el.is_invalid () || el.is_event_number ()) continue;


    bx_muon_edge et = bx_muon_edge(disk_event + sizeof (muon_header_disk_format) + (++i) * sizeof (muon_edge_disk_format));
    if (et.is_last ()) chip++;
    if (et.is_invalid () || et.is_event_number ()) continue;


    bx_muon_raw_hit hit = bx_muon_raw_hit (el, et, board);
    bx_muon_raw_hit::check_t c = hit.check_validity (); 
    if (c == bx_muon_raw_hit::fine) // fine
      raw_hits.push_back (hit);
    else if (c == bx_muon_raw_hit::missing_edge) { // missing edge, skip one and keep the second for next event
      bx_message msg(bx_message::debug, "bx_muon_raw_event: ");
      msg << "event " << u4_trgid << "; mch " << hit.get_muon_channel() << "; missing edge; lead " << el.get_time() << "; trail " << et.get_time() << ";\n" << dispatch;
      i--;
      if (et.is_last ()) chip--; // to correct double counting    
    }
    else {//  i.e. c==bx_muon_raw_hit::inverted: do nothing, skip the hit
      bx_message msg(bx_message::warn, "bx_muon_raw_event: ");
      msg << "event " << u4_trgid << "; mch " << hit.get_muon_channel() << "; inverted time ordering; lead " << el.get_time() << "; trail " << et.get_time() << ";\n" << dispatch;
    }
  } // end of loop on edge pairs
  
  
  if (chip != (constants::muon.tdc.max_boards * constants::muon.tdc.chips_per_board)) {
    bx_message msg(bx_message::critic, "bx_muon_raw_event: ");
    msg << "Inconsitent muon data. Found tdc chips #" << int(chip) << dispatch;
  }
}

/*
 * $Log: bx_muon_event.cc,v $
 * Revision 1.1  2004/04/27 09:41:09  ddangelo
 * Event structure redisegned.
 * bx_reco_event now handles event header and has object for different detector portions.
 * Inheritance from different reconstruction status is kept within each detector portion obj.
 * Hit lists are linked by const ptr (maybe a temporary solution).
 *
 *
 */
