/* BOREXINO Reconstruction program
 *
 * Author: Davide D'Angelo <davide.dangelo@lngs.infn.it>
 *
 * $Id: bx_muon_event.hh,v 1.1 2004/04/27 09:41:09 ddangelo Exp $
 *
 * The muon event object
 * 
 */
#ifndef _BX_MUON_EVENT_HH
#define _BX_MUON_EVENT_HH

#include <vector>

#include "bx_rec_general.hh"
#include "constants.hh"

// ****** raw *************/
class bx_muon_edge {
  public:
    enum slope_t {rising, falling};

    bx_muon_edge (const char *disk_edge);

    unsigned char get_tdc_chip    () const { return (unsigned char)((u4_tdc_word >> 24) & tdc_chip_mask);}
    unsigned char get_tdc_channel () const { return (unsigned char)((u4_tdc_word >> 18) & tdc_channel_mask);}

    slope_t       get_slope       () const { return slope_t((u4_tdc_word) >> 16 & slope_mask); }
    bool          is_overflow     () const { return bool((u4_tdc_word >> 17) & overflow_mask); } 
    bool          is_last         () const { return bool((u4_tdc_word >> 30) & last_datum_mask); }
    bool          is_invalid      () const { return bool((u4_tdc_word >> 31) & invalid_datum_mask); }
    bool          is_event_number () const { return bool((u4_tdc_word >> 23) & event_number_mask); }
    unsigned short get_time        () const { return (unsigned short)( u4_tdc_word); }

  private:
  // bit masks for data decoding
    static const unsigned long event_number_mask;
    static const unsigned long invalid_datum_mask;
    static const unsigned long tdc_chip_mask;
    static const unsigned long tdc_channel_mask;
    static const unsigned long slope_mask;
    static const unsigned long overflow_mask;
    static const unsigned long raw_time_mask;
    static const unsigned long last_datum_mask;

    unsigned long u4_tdc_word;
};

class bx_muon_raw_hit {
  public:
    enum polarity_t {negative=-1, positive=+1}; 
    enum check_t    {fine, missing_edge, inverted};

    bx_muon_raw_hit (const bx_muon_edge& lead, const bx_muon_edge& trail, unsigned char board);
 
    unsigned short get_muon_channel    ()  const { return (u1_tdc_board * constants::muon.tdc.chips_per_board + lead_edge.get_tdc_chip ()) * constants::muon.tdc.channels_per_chip + lead_edge.get_tdc_channel (); } // 0 based
    unsigned short get_logical_channel ()  const { return get_muon_channel () + constants::muon.channel_offset + 1; } // 1 based

    unsigned long  get_lead_time        () const {return lead_edge.get_time(); }
    unsigned long  get_trail_time       () const {return trail_edge.get_time(); }
    unsigned long  get_time_diff        () const {return lead_edge.get_time() - trail_edge.get_time(); }
    polarity_t     get_polarity         () const {return (lead_edge.get_slope() == bx_muon_edge::rising) ? positive : negative; }

    check_t        check_validity       () const;
  
    bool is_integrity                   () const {return bool(!(get_muon_channel() % constants::muon.tdc.channels_per_chip)); }
    bool is_led_reference               () const {return bool(get_muon_channel() == led_reference_channel);}
    bool is_yellow_pulse                () const {return bool(get_muon_channel() == yellow_pulse_channel);}
    bool is_clock_pulse                 () const {return bool(get_muon_channel() == clock_pulse_channel);}
    bool is_omt_pulse                   () const {return bool(get_muon_channel() == omt_pulse_channel);}
    bool is_special_channel             () const {return bool((lead_edge.get_tdc_chip() == 4) && (lead_edge.get_tdc_channel() > 15));}
    bool is_pmt_channel                 () const {return (!is_integrity()) && (!is_special_channel());}

  private:
    static const unsigned short led_reference_channel;
    static const unsigned short yellow_pulse_channel;
    static const unsigned short clock_pulse_channel;
    static const unsigned short omt_pulse_channel;

    bx_muon_edge lead_edge;
    bx_muon_edge trail_edge;
    unsigned char u1_tdc_board;
};
   

class bx_muon_raw_event {
  public:
    bx_muon_raw_event (const char *disk_event);
    virtual ~bx_muon_raw_event () {}
    const bx_muon_raw_hit& get_raw_hit (int i) const { return raw_hits[i]; }
    unsigned long get_raw_nhits    ()      const {return raw_hits.size();}
    unsigned long get_nedges       ()      const {return u4_nedges;}
    unsigned long get_trgid        ()      const {return u4_trgid;}
    bool get_error_flag            ()      const {return b_error_flag;}
  private:
    unsigned long u4_nedges;
    unsigned long u4_trgid;
    bool b_error_flag;
    typedef std::vector<bx_muon_raw_hit> bx_muon_raw_hit_vector;
    bx_muon_raw_hit_vector raw_hits;
};

// ****** decoded *************/
class bx_muon_decoded_hit {
  public:
    bx_muon_decoded_hit ( float time, float charge, const bx_muon_raw_hit* raw_hit_ptr) : f4_time(time), f4_charge(charge), raw_hit(raw_hit_ptr) {}
    const bx_muon_raw_hit* get_raw_hit () const {return raw_hit;}

    float get_time          () const { return f4_time; }
    float get_charge        () const { return f4_charge; }

  private:
    float f4_time, f4_charge;
    const bx_muon_raw_hit* raw_hit;
}; 

class bx_muon_decoded_event {
  public:
    bx_muon_decoded_event () {}
    virtual ~bx_muon_decoded_event () {}

    const bx_muon_decoded_hit& get_decoded_hit (int i) const { return decoded_hits[i]; }
    unsigned long get_decoded_nhits ()            const { return decoded_hits.size(); }
    unsigned long get_decoded_nhits (int channel) const { return nhits_per_channel[channel]; }
    
  private:
    unsigned long nhits_per_channel[256]; // to be replaced as soon as constants is improved

    typedef std::vector<bx_muon_decoded_hit> bx_muon_decoded_hit_vector;
    bx_muon_decoded_hit_vector decoded_hits;

  friend class bx_muon_decoder;
};

class bx_muon_event: 
  public bx_muon_raw_event, 
  public bx_muon_decoded_event {
  public:
    bx_muon_event (const char *disk_event): bx_muon_raw_event(disk_event) {}
    virtual ~bx_muon_event () {}
};

#endif
/*
 * $Log: bx_muon_event.hh,v $
 * Revision 1.1  2004/04/27 09:41:09  ddangelo
 * Event structure redisegned.
 * bx_reco_event now handles event header and has object for different detector portions.
 * Inheritance from different reconstruction status is kept within each detector portion obj.
 * Hit lists are linked by const ptr (maybe a temporary solution).
 *
 *
 */
