/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: laben_time_hit.cc,v 1.3 2004/04/26 13:53:22 razeto Exp $
 *
 * laben_time_hit implementation
 *
 */
#include "bx_dbi.hh"
#include "db_run.hh"
#include "db_profile.hh"
#include "constants.hh"
#include "laben_time_hit.hh"

const float laben_time_hit::ramp_length_ns = 50.;
const float laben_time_hit::window_upper_bound = 15.;
const float laben_time_hit::window_lower_bound = 15.;
const unsigned char laben_time_hit::spire_threshold = 10;
const float laben_time_hit::d80 = 80.;

laben_time_hit::laben_time_hit (const bx_laben_raw_hit& hit) {

    // Get some parameters from the db
  int lg = hit.get_channel ();
  const db_run& run_info = bx_dbi::get()->get_run (bx_dbi::get ()->get_current_run_number ());
  const db_profile& profile_info = bx_dbi::get()->get_profile (run_info.get_profile_id ());
  crate_time_correction = profile_info.laben_crate_delay (constants::crate (lg));
  i_low = run_info.get_laben_precalib_low_bin (lg);
  i_high = run_info.get_laben_precalib_high_bin (lg);
    // The following parameters can not be in bd_run since laben_time_hit::guess_slope can 
    // be called during precalib. If not present use the default.
  if (run_info.check_laben_precalib_delta80 (lg)) f_d80 = run_info.get_laben_precalib_delta80 (lg);
  else f_d80 = d80;
  if (run_info.check_laben_precalib_rising_on_even (lg)) rising_on_even = run_info.get_laben_precalib_rising_on_even (lg);
  else rising_on_even = laben_time_hit::rising;
  if (run_info.check_laben_precalib_gray_shift (lg)) i_gr_shift = run_info.get_laben_precalib_gray_shift (lg);
  else i_gr_shift = 0;
  
    // Store the time data from the hit
  u1_ramp_1 = hit.get_time_1 ();
  u1_ramp_2 = hit.get_time_2 ();
  u2_gray_count = hit.get_gray_counter () - i_gr_shift;
  i_hw_slope = laben_time_hit::ramp_slope((rising_on_even == laben_time_hit::rising) ? u2_gray_count : ~u2_gray_count & 0x1);

    // Some constants used later
  f_ramp_slope = (i_high - i_low) / ramp_length_ns;
  f_t20 = 2 * ramp_length_ns - f_d80;
  f_t30 = f_d80 - ramp_length_ns;
} 

float laben_time_hit::get_time () {
  
    // If no predecoding info are loaded for this channel no calculations can be done, return 0.
  if (i_low >= i_high) {
    f_error = 1.;
    return 0.;
  }

    // Use guess_slope returned slope instead of hw_slope. 
  laben_time_hit::ramp_slope slope = guess_slope ();

    // Calculate the 50ns relative time of the the 2 hits
  float t1, t2;
  m_calc_ramp_times (slope, t1, t2);

    // Calculate the gray counter time
  float time_gray = ramp_length_ns * u2_gray_count;
    // If guessed slope and hw slope does not coincide suppose the gray counter is late.
  if (slope != i_hw_slope) time_gray += ramp_length_ns;

    // If sample 1 is to close to spires use sample 2
  if (u1_ramp_1 < (i_low + spire_threshold) || u1_ramp_1 > (i_high + spire_threshold))
    return t2 - f_d80 + time_gray + crate_time_correction;
    // Else use sample 1 (which is more accurate than d80)
  return t1 + time_gray + crate_time_correction;
}

void laben_time_hit::m_calc_ramp_times (laben_time_hit::ramp_slope slope, float& t1, float& t2) {
  if (slope == laben_time_hit::rising) {
    t1 = (u1_ramp_1 - i_low) / f_ramp_slope;
    if (t1 < f_t20) t2 = ramp_length_ns + (i_high - u1_ramp_2) / f_ramp_slope;
    else t2 = 2 * ramp_length_ns + (u1_ramp_1 - i_low) / f_ramp_slope;
  } else {
    t1 = (i_high - u1_ramp_1) / f_ramp_slope;
    if (t1 < f_t20) t2 = ramp_length_ns + (u1_ramp_1 - i_low) / f_ramp_slope;
    else t2 = 2 * ramp_length_ns + (u1_ramp_1 - i_low) / f_ramp_slope;
  }
}

float laben_time_hit::get_d80 () {
  
    // If no predecoding info are loaded for this channel no calculations can be done, return 0.
  if (i_low >= i_high) {
    f_error = 1.;
    return 0.;
  }

    // Use guess_slope returned slope instead of hw_slope. 
  laben_time_hit::ramp_slope slope = guess_slope ();

    // Calculate the 50ns relative time of the the 2 hits
  float t1, t2;
  m_calc_ramp_times (slope, t1, t2);

  return t2 - t1;
}

laben_time_hit::ramp_slope laben_time_hit::guess_slope () {
  
    // If no predecoding info are loaded for this channel use the graycounte parity
  if (i_low >= i_high) {
    f_error = 1.;
    return i_hw_slope;
  }
  
    // Sistances from the second sample for the 2 projections
  float d_rising, d_falling;

    // Rising ramp  
  float t = (u1_ramp_1 - i_low) / f_ramp_slope; 	// the time of the first sample
  if (t < f_t20) d_rising = ::fabs (i_high - (t + f_t30) * f_ramp_slope - u1_ramp_2);
  else d_rising = ::fabs (i_low + (t - f_t20) * f_ramp_slope - u1_ramp_2);

    // Falling ramp
  t = ramp_length_ns - t;			// the time of the first sample
  if (t < f_t20) d_falling = ::fabs (i_low + (t + f_t30) * f_ramp_slope - u1_ramp_2);
  else d_falling = ::fabs (i_high + (t - f_t20) * f_ramp_slope - u1_ramp_2);

    // Calculate the error and the slope
  float d_min, d_max;
  laben_time_hit::ramp_slope guess_slope;
  if (d_falling > d_rising) { 
    d_max = d_falling;
    d_min = d_rising;
    guess_slope = laben_time_hit::rising;
  } else {
    d_min = d_falling;
    d_max = d_rising;
    guess_slope = laben_time_hit::falling;
  }
  if (d_max == 0.) f_error = 1.;	   // if max = 0 even min = 0 so the error is maximal
  else f_error = d_min / ( d_min + d_max); // else use the pondered distance
  
    // If the one and only if min is in the lower bound and max is > the the upper bound use
    // the found slope
  if (d_min < window_lower_bound && d_max > window_upper_bound) return guess_slope;
  else return i_hw_slope;
} 

/*
 * $Log: laben_time_hit.cc,v $
 * Revision 1.3  2004/04/26 13:53:22  razeto
 * Updated to follow new name/syntax of db_run and db_profile
 *
 * Revision 1.2  2004/04/24 17:37:40  razeto
 * Added a method for calculating d80
 *
 * Revision 1.1  2004/04/15 13:00:14  razeto
 * Added
 *
 */
