/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: bx_dbi.cc,v 1.6 2004/04/26 14:18:05 razeto Exp $
 *
 * Implementation of bx_dbi
 *
 */
#include "bx_dbi.hh"
#include "db_run.hh"
#include "db_profile.hh"

#include <iostream>
#include <fstream>
#include <sstream>

bx_dbi *bx_dbi::me = 0;

bx_dbi* bx_dbi::get () {
  if (!me) me = new bx_dbi;
                                                                                                                                    
  return me;
}


bx_dbi::bx_dbi () {

    // Set the enumerator/name corrispondence
  hash_name_indexes[daq_config] = "DAQCONFIGDBNAME";
  hash_name_indexes[bx_geometry] = "BXGEOMETRYDBNAME";
  hash_name_indexes[bx_calib] = "BXCALIBDBNAME";
  hash_name_indexes[bx_physics] = "BXPHYSICSDBNAME";
  hash_name_indexes[bx_slow] = "BXSLOWDBNAME";
  hash_name_indexes[channelhistory] = "CHHISTORYDBNAME";
  
    // Set default values
  params["DAQCONFIGDBNAME"] = "daq_config";
  params["BXGEOMETRYDBNAME"] = "bx_geometry";
  params["BXCALIBDBNAME"] = "bx_calib";
  params["BXPHYSICSDBNAME"] = "bx_physics";
  params["BXSLOWDBNAME"] = "bx_slow";
  params["CHHISTORYDBNAME"] = "channelhistory";
  params["PGUSER"] = "borex_guest";
  params["SERVER1"] = "bxdb";
  params["PORT1"] = 5432;
  params["SERVER2"] = "bxdb.lngs.infn.it";
  params["PORT2"] = 5432;

  m_initialize_from_file ("bx_dbconfig.inp");

  m_open_connections (daq_config);
  m_open_connections (bx_geometry);
  m_open_connections (bx_calib);
  m_open_connections (bx_physics);
  m_open_connections (channelhistory);
}


void bx_dbi::m_initialize_from_file (const std::string &filename) {
  std::ifstream in (filename.c_str ());
  if (!in) get_message (bx_message::critic) << "cannot open " << filename << " config file" << dispatch;
 
  std::string line;
  std::string::size_type pos;
  while (in) {
      // remove white spaces
    in >> std::ws;
     
      // get line
    if (!std::getline (in, line)) break;

      // erase inline comments
    pos = line.find ('#');
    if (pos != std::string::npos) line.erase (pos);
                                                                                                                                        
      // skip blank line
    if (line.empty ()) continue;
                                                                                                                                        
      // look for space
    pos = line.find (':');
    if (pos == std::string::npos) get_message (bx_message::critic) << "syntax error in config file: " << line << dispatch;

    std::string name = line.substr (0, pos);
    std::string value = line.substr (pos + 1);
                                                                                                                                        
      // skip white spaces again
    std::istringstream tmp(value);
    tmp >> std::ws;
    tmp >> value;

    // set value in params table
    params[name] = value;
    get_message (bx_message::debug) << "setting " << name << " to " << value << dispatch;
  }
}


void bx_dbi::m_open_connections (database_id id) {

  const char *ip = params["SERVER1"].get_string ().c_str ();
  const char *db = params[hash_name_indexes[id]].get_string ().c_str ();
  const char *user = params["PGUSER"].get_string ().c_str (); 

  const char *passwd = "";
  if (params.check ("PGPASSWD")) passwd = params["PGPASSWD"].get_string ().c_str ();

  long int port_i = params["PORT1"].get_int ();
  
  get_message (bx_message::debug) << "PQsetdbLogin (" << ip << "," << port_i << ", 0, 0, " << db << ", " << user << "," << passwd << ")" << dispatch;

  std::ostringstream s_port;
  s_port << port_i;
  PGconn *connection = PQsetdbLogin (ip, s_port.str ().c_str (), 0, 0, db, user, passwd);


  if (PQstatus (connection) == CONNECTION_BAD) {
    const char *ip2 = params["SERVER2"].get_string ().c_str ();
    long int port_i2 = params["PORT2"].get_int ();

    get_message (bx_message::debug) << "PQsetdbLogin (" << ip2 << "," << port_i2 << ", 0, 0, " << db << ", " << user << "," << passwd << ")" << dispatch;

    std::ostringstream s_port2;
    s_port2 << port_i2;
    connection = PQsetdbLogin (ip2, s_port.str ().c_str (), 0, 0, db, user, passwd);
  }
  
  if (PQstatus (connection) == CONNECTION_BAD) 
    get_message (bx_message::critic) << "cannot connect to both databases " <<  params["SERVER1"] << " and " << params["SERVER2"] << dispatch;

  connections[id] = connection;
}


const bx_dbi::table& bx_dbi::query (database_id db_id, const std::string& from_table, const std::string& where,
			      const std::string &table_fields, const std::string& order, int max_lines) {
  std::ostringstream sql_query;
  
  sql_query << "SELECT " << table_fields << " FROM " << from_table;
  if (where.size ()) sql_query << " WHERE " << where;
  if (order.size ()) sql_query << " ORDER BY " << order;
  
  return query (db_id, sql_query.str (), max_lines);
}

const bx_dbi::table& bx_dbi::query (database_id db_id, const std::string& sql_query, int max_lines) {
  const std::string &db_name = params[hash_name_indexes[db_id]].get_string ();
  query_result.clear ();
  
  
  PGresult *result = PQexec (connections[db_id], sql_query.c_str ());

  if (PQresultStatus (result) != PGRES_TUPLES_OK) 
    get_message (bx_message::critic) << "query \"" << sql_query << "\" on database \"" << db_name << "\" failed with error \"" << PQresultErrorMessage (result) << "\"" << dispatch;

  int tuples = PQntuples (result);
  if (!tuples) {
    get_message (bx_message::warn) << "no tuples found for query \"" << sql_query << "\" on database \"" << db_name << "\"" << dispatch;
    return query_result;
  }

  if (max_lines < 0) {
    max_lines *= -1;
    if (tuples > max_lines) 
      get_message (bx_message::critic) << "query \"" <<  sql_query << "\" on database \"" << db_name + "\" returned " << tuples << " tuples while only " << max_lines << " were admitted" << dispatch;
  }

  int lines = max_lines < tuples ? max_lines : tuples;
  if (!max_lines) lines = tuples;

  int fields = PQnfields (result);

  
  
  for (int i = 0; i < fields; i++) {
    const char *field = PQfname (result, i);
    for (int j = 0; j < lines; j++) {
      const char *value = PQgetvalue (result, j, i);
      query_result[field].push_back (std::string (value));
    }
  }
      
  return query_result;
}

db_run& bx_dbi::get_run_nonconst (int run_number) {
  if (!run_map.check (run_number)) run_map[run_number] = new db_run(run_number);
  return *(run_map[run_number]);
}

db_profile& bx_dbi::get_profile_nonconst (int profile_id) {
  if (!profile_map.check (profile_id)) profile_map[profile_id] = new db_profile(profile_id);
  return *(profile_map[profile_id]);
}		    

bx_message& bx_dbi::get_message (bx_message::message_level level) {
  message.clear ();
  message.set_level (level);
  
  if (params.check ("log_level")) message.set_log_level (params["log_level"]);
  if (params.check ("print_level")) message.set_print_level (params["print_level"]);

  message << "bx_bdi: ";
  return message;
}
/*
 * $Log: bx_dbi.cc,v $
 * Revision 1.6  2004/04/26 14:18:05  razeto
 * Bug still not found, but fixed using port_i
 *
 * Revision 1.5  2004/04/26 13:49:17  razeto
 * Removed db_run/db_profile header include in bx_dbi.hh
 *
 * Revision 1.4  2004/04/09 07:56:48  razeto
 * Changed the enum names for some dbs to a more familiar name: moved the enum in the class local namespace.
 * Fixed the sql query build syntax in query method.
 * Exported get_message.
 * Added db_run and db_profile getters.
 *
 * Revision 1.3  2004/04/05 13:14:45  razeto
 * Used messages for expcetions
 *
 * Revision 1.2  2004/04/03 09:25:46  razeto
 * Added messenger
 *
 * Revision 1.1  2004/04/01 13:02:04  razeto
 * Added bx_dbi
 *
 */
