/* BOREXINO Reconstruction program
 * 
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: bx_options.cc,v 1.7 2004/04/24 17:40:22 razeto Exp $
 *
 * Implementation of bx_options
 * Here bx_message messages are not used and direct throw are called on error,
 * this is done because every error is a line option/config file parsing error 
 * and have to be reported directly.
 *
 */
#include "bx_options.hh"
#include "bx_module_factory.hh"
#include "messenger.hh"

#include <stdexcept>
#include <iostream>
#include <fstream>
#include <sstream>
#include <unistd.h>
#include <time.h>


bx_options *bx_options::me = 0;


bx_options* bx_options::get () {
  if (!me) me = new bx_options;

  return me;
}

bx_options::bx_options (): s_config_filename("echidna.cfg"), s_module_list_filename("modules.cfg") {
  time_t t;
  t = time (0);
  
  char time_string[31];
  ::strftime(time_string, 31, "echidna_%d:%m:%Y_%H%M%S.log", ::localtime(&t));

  s_log_filename = time_string;
}

void bx_options::set_module_dot_parameter_value (const std::string& module_dot_parameter, const std::string& value) {
  std::string::size_type dot_position = module_dot_parameter.find (".");
  if (dot_position == std::string::npos || dot_position == 0) 
    throw std::runtime_error ("set_module_dot_parameter_value require module.parameter syntax (" + module_dot_parameter + ")");

  //std::cout << "Setting " << module_dot_parameter << " to " << value << std::endl;
  
  std::string module_name = module_dot_parameter.substr (0, dot_position);
  std::string parameter_name = module_dot_parameter.substr (dot_position + 1);

  bx_base_module *ptr = bx_module_factory::get ()->get_module (module_name);

  ptr->set_parameter (parameter_name, value);
}


void bx_options::set_command_line (int argc, char * const argv[]) {
  extern char *optarg;
  extern int optind, opterr, optopt;

  bool confing_file_specified = false;
  int verbosity = 0;

  opterr = 0;
  while (1) {
    const char *valid_options = ":vc:C:f:n:p:lo:";
    int c = ::getopt (argc, argv, valid_options);
    if (c == -1) break;
    std::ostringstream opt;
    opt << "\"-" << char (optopt) << "\"";
    switch (c) {
      case 'c':
	s_config_filename = optarg;
	confing_file_specified = true;
	break;
      case 'C':
	s_module_list_filename = optarg;
	break;
      case 'f':
	m_schedule_set_module_dot_parameter_value ("bx_event_reader.file_url", optarg);	
	break;
      case 'p':
	  // optind is the index to the next option (so argv[optind] is the second opt parameter)
	if (!argv[optind] || argv[optind][0] == '-') throw std::runtime_error ("option -p requires 2 parameters with the \"module.param_name value\" syntax"); 
	m_schedule_set_module_dot_parameter_value (optarg, argv[optind++]);
	break;
      case 'l':
	s_log_filename = "";
        if (optind < argc && argv[optind] && argv[optind][0] != '-') 
	    s_log_filename = argv[optind++];
	break;
      case 'v':
	verbosity ++;
	break;
      case 'o':
	m_schedule_set_module_dot_parameter_value ("bx_event_writer.root_file_name", optarg);	
	break;
      case ':':
        throw std::runtime_error ("missing value on command line option " + opt.str ());
	break;
      default:
        throw std::runtime_error ("unknown command line option: " + opt.str ());
    }
  }

  if (optind < argc) {
    std::ostringstream msg;
    msg << "unknown fields on the command line:";
    while (optind < argc) msg << " \"" << argv[optind++] << "\"";
    throw std::runtime_error (msg.str ());
  }

  if (verbosity == 1) bx_message::set_default_print_level (bx_message::info);
  else if (verbosity > 1) bx_message::set_default_print_level (bx_message::debug);
  else bx_message::set_default_print_level (bx_message::warn);

  m_read_config_file (confing_file_specified);

  m_update_module_dot_parameter_value ();
}

void bx_options::m_read_config_file (bool error_on_not_found) {
  std::ifstream in (s_config_filename.c_str ());
  if (!in) {
    if (error_on_not_found) std::runtime_error ("cannot open " + s_config_filename + " config file");
    else return;
  }

  std::string line;
  std::string::size_type pos;
  while (in) {
      // remove white spaces
    in >> std::ws;
    
      // get line
    if (!std::getline (in, line)) break;

      // erase inline comments
    pos = line.find ('#');
    if (pos != std::string::npos) line.erase (pos);

      // skip blank line
    if (line.empty ()) continue;
    
      // look for space
    pos = line.find (' ');
    if (pos == std::string::npos) throw std::runtime_error("syntax error in config file: " + line);

    std::string module_dot_parameter = line.substr (0, pos);
    std::string value = line.substr (pos + 1);
    
      // skip white spaces again
    std::istringstream tmp(value);
    tmp >> std::ws;
    tmp >> value;
    
    // schedule set operation
    m_schedule_set_module_dot_parameter_value (module_dot_parameter, value);
  }
}
    
void bx_options::m_schedule_set_module_dot_parameter_value (const std::string& module_dot_parameter, const std::string& value) {
  scheduled_sets.push_back (scheduled_parameter_set(module_dot_parameter, value));
}

void bx_options::m_update_module_dot_parameter_value () {
  for (int i = scheduled_sets.size () - 1; i >= 0; i--) 
    set_module_dot_parameter_value (scheduled_sets[i].s_module_dot_parameter, scheduled_sets[i].s_value);
}
/*
 * $Log: bx_options.cc,v $
 * Revision 1.7  2004/04/24 17:40:22  razeto
 * Updated to the new bx_reader file access
 *
 * Revision 1.6  2004/04/18 09:16:52  razeto
 * Added -o option, to select the output root file
 *
 * Revision 1.5  2004/04/12 16:07:39  razeto
 * Added configurable default print level for messages
 *
 * Revision 1.4  2004/04/06 12:39:28  razeto
 * Added logfile handling
 *
 * Revision 1.3  2004/04/05 13:12:27  razeto
 * Updated a comment, changed the behaviour if no user option file is present
 *
 * Revision 1.2  2004/03/29 13:17:30  razeto
 * Updated bx_options to the vdt usage
 *
 * Revision 1.1  2004/03/26 16:36:11  razeto
 * Introduced bx_options; a lot of code modified to read options and parameters.
 * Bx_event_reader interface changed: now there is a standard constructor and
 * the file opening is done at begin using the parameters for the file name.
 *
 *
 */
