/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: db_profile.cc,v 1.3 2004/04/26 13:48:29 razeto Exp $
 *
 * The database interface for profile informations
 *
 */
#include "db_profile.hh"
#include "bx_dbi.hh"
#include "messenger.hh"
#include "date.hh"
#include "messenger.hh"

#include <sstream>

db_profile::db_profile (int profile_id) {

  channel_description_type_map["Ordinary"] = db_profile::ordinary;
  channel_description_type_map["Trigger"] = db_profile::trigger;
  channel_description_type_map["Laser"] = db_profile::laser;
  
  channel_description_type_map["Pulser"] = db_profile::pulser;
  
  channel_description_type_map["NotInstalled"] = db_profile::empty;
  channel_description_type_map["TotalSum"] = db_profile::total_sum;
  channel_description_type_map["AttenuatedSum"] = db_profile::attenuated_sum;
  channel_description_type_map["TriggerRamp"] = db_profile::trigger_ramp;
  channel_description_type_map["TriggerPulse"] = db_profile::trigger;
  channel_description_type_map["LaserPulse"] = db_profile::laser;

  
  m_load_laben_channel_mapping (profile_id);  
  m_load_fadc_channel_mapping (profile_id);  
  m_load_muon_channel_mapping (profile_id);  

  m_load_profile_table (profile_id);

  m_load_detector_geometry (profile_id);

  m_load_physics_constants (profile_id);
  laben_crate_delay_v[1]  = 4.8;
  laben_crate_delay_v[2]  = 4.7;
  laben_crate_delay_v[3]  = 3.7;
  laben_crate_delay_v[4]  = 3.7;
  laben_crate_delay_v[5]  = 1.9;
  laben_crate_delay_v[6]  = 2.7;
  laben_crate_delay_v[7]  = 0.;
  laben_crate_delay_v[8]  = 3.1;
  laben_crate_delay_v[9]  = 4.5;
  laben_crate_delay_v[10] = 4.5;
  laben_crate_delay_v[11] = -7.7;
  laben_crate_delay_v[12] = 2.4;
  laben_crate_delay_v[13] = 4.7;
  laben_crate_delay_v[14] = 4.;


  bx_dbi::get ()->get_message (bx_message::info) << "db_profile: db_profile (" << profile_id << ") initialized" << dispatch;
}

void db_profile::m_load_laben_channel_mapping (int profile_id) {
  bx_dbi *dbi = bx_dbi::get ();
  bx_message &msg = bx_dbi::get ()->get_message (bx_message::critic);
  msg << "db_profile: ";

  std::ostringstream t;
  t << "\"ProfileID\"=" << profile_id << " AND \"ChannelID\">0";

  const bx_dbi::table &table = dbi->query (bx_dbi::daq_config, "\"LabenChannelMapping\"", t.str (), "\"ChannelID\",\"ChannelDescription\"", "\"ChannelID\"", 0);

  if (!table.size ()) msg << "no LabenChannelMapping entries for profile " << profile_id << dispatch;

  const bx_dbi::column& ch_id = table["ChannelID"];
  const bx_dbi::column& ch_descr = table["ChannelDescription"];
  
  channel_description_type_map["Ordinary"] = db_profile::empty; // this is an hack since the db has ordinary even for
 								// channels without pmt. The channel will be set to ordinary
								// later while loading bx_geometry.
  for (unsigned long i = 0; i < ch_id.size (); i++) {
    if (!channel_description_type_map.check(ch_descr[i].get_string ())) msg << "unknown channel description " << ch_descr[i] << dispatch;
    else logical_channel_description_v[ch_id[i].get_int ()] = channel_description_type_map[ch_descr[i].get_string ()];
  }  
  channel_description_type_map["Ordinary"] = db_profile::ordinary; // restore the map


  const bx_dbi::table &table2 = dbi->query (bx_dbi::bx_geometry, "\"HolesMapping\"", t.str (), "\"ChannelID\",\"HoleLabel\",\"X\",\"Y\",\"Z\",\"Conc\"", "\"ChannelID\"", 0);

  if (!table2.size ()) msg << "no HolesMapping entries for profile " << profile_id << dispatch;
  const bx_dbi::column& ch_id2 = table["ChannelID"];
  const bx_dbi::column& hole = table["HoleLabel"];
  const bx_dbi::column& conc = table["Conc"];
  const bx_dbi::column& x = table["X"];
  const bx_dbi::column& y = table["Y"];
  const bx_dbi::column& z = table["Z"];
  for (unsigned long i = 0; i < ch_id.size (); i++) {
    int lg = ch_id2[i].get_int ();
    pmt_has_cone_v[lg] = conc[i].get_int ();
    pmt_hole_id_v[lg] = hole[i].get_int ();
    pmt_coordinates c;
    c.x = x[i].get_float ();
    c.y = y[i].get_float ();
    c.z = z[i].get_float ();
    pmt_coordinates_v[lg] = c;
    logical_channel_description_v[lg] = db_profile::ordinary; // this ends the hack described above
  }
  i_number_of_internal_pmt = ch_id.size ();

  dbi->get_message (bx_message::debug) << "db_profile: load_laben_channel_mapping OK" << dispatch;
}


void db_profile::m_load_fadc_channel_mapping (int profile_id) {
  bx_dbi *dbi = bx_dbi::get ();
  bx_message &msg = dbi->get_message (bx_message::critic);
  msg << "db_profile: ";
  
  std::ostringstream t;
  t << "\"ProfileID\"=" << profile_id << " AND \"FADCChannelID\">0";

  const bx_dbi::table &table = dbi->query (bx_dbi::daq_config, "\"SumsChannelMapping\"", t.str (), "\"FADCChannelID\",\"ChannelDescription\"", "\"FADCChannelID\"", 0);   
  
  const bx_dbi::column& ch_id = table["FADCChannelID"];
  const bx_dbi::column& ch_descr = table["ChannelDescription"];
  for (unsigned long i = 0; i < ch_id.size (); i++) {
    if (!channel_description_type_map.check(ch_descr[i].get_string ())) msg << "unknown channel description " << ch_descr[i] << dispatch;
    else logical_channel_description_v[ch_id[i].get_int ()] = channel_description_type_map[ch_descr[i].get_string ()];
  }  
  
  // to load from bx_geometry
  // unknow syntax
  // "select distinct h.\"ChannelID\", s.\"FADCChannelID\" from \"SumsCableMapping\" s, \"HolesMapping\" h, \"CableMapping\" c where h.\"ProfileID\"=s.\"ProfileID\" and s.\"ProfileID\"=c.\"ProfileID\" and h.\"ProfileID\"=%d and s.\"FEBoard\"=c.\"FEBoard\" and s.\"RackID\"=c.\"RackID\" and h.\"HoleLabel\"=c.\"HoleLabel\"
  dbi->get_message (bx_message::debug) << "db_profile: load_fadc_channel_mapping OK" << dispatch;
}


void db_profile::m_load_muon_channel_mapping (int profile_id) {
  bx_dbi *dbi = bx_dbi::get ();

  dbi->get_message (bx_message::debug) << "db_profile: load_muon_channel_mapping OK" << dispatch;
}

void db_profile::m_load_detector_geometry (int profile_id) {
  bx_dbi *dbi = bx_dbi::get ();
  bx_message &msg = dbi->get_message (bx_message::critic);
  msg << "db_profile: ";

  std::ostringstream t;
  t << "\"ProfileID\"=" << profile_id;
  const bx_dbi::table &detector_params = dbi->query (bx_dbi::bx_geometry, "\"DetectorGeometry\"", t.str (), "*", "", -1);

  f_stainless_steel_sphere_radius = detector_params["SSSRadius"][0].get_float ();
  f_inner_vessel_radius = detector_params["InnerVesselRadius"][0].get_float ();
  f_light_collector_radius = f_stainless_steel_sphere_radius - detector_params["LGEntranceOffset"][0].get_float ();
  f_light_guide_exit_aperture = detector_params["LGExitRadius"][0].get_float ();
  f_light_guide_entry_aperture = detector_params["LGEntryRadius"][0].get_float ();
  f_light_guide_lenght = detector_params["LGLength"][0].get_float ();
  f_pmt_chathode_radius = detector_params["CathodeRadius"][0].get_float ();
  f_pmt_bulb_radius = detector_params["BulbRadius"][0].get_float ();

  dbi->get_message (bx_message::debug) << "db_profile: m_load_detector_geometry OK" << dispatch;
}


void db_profile::m_load_profile_table (int profile_id) {
  bx_dbi *dbi = bx_dbi::get ();
  bx_message &msg = dbi->get_message (bx_message::critic);
  msg << "db_profile: ";

  std::ostringstream t;
  t << "\"ProfileID\"=" << profile_id;
  const bx_dbi::table &profile_params = dbi->query (bx_dbi::daq_config, "\"Profiles\"", t.str (), "*", "", -1);

  i4_neutrino_trigger_tag = profile_params["NeutrinoTrigger"][0].get_int ();
  i4_muon_mtb_trigger_tag = profile_params["MuonMTBTrigger"][0].get_int ();
  i4_muon_tct_trigger_tag = profile_params["MuonTCTTrigger"][0].get_int ();
  i4_laser266_trigger_tag = profile_params["Laser266Trigger"][0].get_int ();
  i4_laser355_trigger_tag = profile_params["Laser355Trigger"][0].get_int ();
  i4_laser394_trigger_tag = profile_params["Laser394Trigger"][0].get_int ();
  i4_pulser_trigger_tag = profile_params["CalibPulserTrigger"][0].get_int ();
  i4_random_trigger_tag = profile_params["RandomTrigger"][0].get_int ();
  f_trigger_start_gate = profile_params["StartGateTrigger"][0].get_float ();
  f_trigger_end_gate = profile_params["EndGateTrigger"][0].get_float ();
  f_laser394_trigger_start_gate = profile_params["StartGateLaser"][0].get_float ();
  f_laser394_trigger_end_gate = profile_params["EndGateLaser"][0].get_float ();

  dbi->get_message (bx_message::debug) << "db_profile: load_profile_table OK" << dispatch;
}

void db_profile::m_load_physics_constants (int profile_id) {
  bx_dbi *dbi = bx_dbi::get ();

  std::ostringstream t;
  t << "\"ProfileID\"=" << profile_id;
  const bx_dbi::table &physics_params = dbi->query (bx_dbi::bx_physics, "\"FixedParameters\"", t.str (), "*", "", -1);

  f_pmt_time_jitter = physics_params["PMTTimeJitter"][0].get_float ();
  f_pmt_reflectivity = physics_params["PMTReflCoeff"][0].get_float ();
  f_light_guide_reflectivity = physics_params["LGExterCoeff"][0].get_float ();
  f_sss_specular_reflectivity = physics_params["SSSSpecRefl"][0].get_float ();
  f_sss_diffusion_reflectivity = physics_params["SSSDiffRefl"][0].get_float ();
  f_sss_uni_reflectivity = physics_params["SSSUnifRefl"][0].get_float ();

  dbi->get_message (bx_message::debug) << "db_profile: m_load_physics_constants OK" << dispatch;
}
/*  
 *  $Log: db_profile.cc,v $
 *  Revision 1.3  2004/04/26 13:48:29  razeto
 *  Added db_acl to check set calls for calling module to have the right privileges.
 *  Fixed the names of set/get methods.
 *  Modifications decided at the software Paris meeting.
 *
 *  Revision 1.2  2004/04/12 16:08:19  razeto
 *  Added crate delay fixed map
 *
 *  Revision 1.1  2004/04/09 08:05:00  razeto
 *  Added
 *
 */
