/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: messenger.hh,v 1.7 2004/04/12 16:07:40 razeto Exp $
 *
 * 2 classes are defined: 
 *   messenger: which is the singleton which receives messages and dump
 *   them on monitor and on logfile
 *   bx_message: which is the message created from modules to diagnose
 *   something. They are ostream, so the standard cout syntax (<<) can be
 *   used. A specialized dispatch manipulator has to be called at the
 *   end of the message to dispatch it to the messenger.
 *   Module usage is simplified by the bx_base_module::get_message(message_level)
 *   which fills all the level fields.
 *   For other uses an istance of bx_message has to be initialized using
 *   set_level and set_*_level.
 *   A message is printed if its level is >= than the print_level (idem for log).
 *   
 */
#ifndef _MESSANGER_H
#define _MESSANGER_H
#include <string>
#include <sstream>
#include <fstream>

#include "cmap.hh"
#include "vdt.hh"

class bx_message;

class messenger {
  public:
    static messenger* get ();

    void submit (bx_message &);
    void flush () { if (log) log.close (); }
  private:
    messenger ();
    static messenger *me;
    std::ofstream log;
};

  // a bx_message inherits from ostringstream, so it is a stream similar to cout,
  // then the message text can be written with << operators.
  // Alternativelly a () oerator is present so that the message text can be set
  // whith bx_message msg; msg("text").dispatch();
class bx_message: public std::ostringstream {
  public:
    enum message_level { 
      none = 0,
      debug = 1,
      info = 2,
      warn = 3,
      error = 4,
      critic = 5,
    };

    bx_message (message_level level = none, const std::string& initial_message = "");
    static void set_default_print_level (message_level level) { default_print_level = level; }
    
      // Clear the message both text and levels
    void clear () { clear_text (); i4_print_level = default_print_level; i4_log_level = default_log_level; i4_message_level = none; } 
      // Clear only the text
    void clear_text () { str(""); }

      // Set the message text (equivalent to message << str) 
    bx_message& operator() (const std::string& str) { return (bx_message &)(*this << str); } // no need to intelligent cast since
    											     // this is known to be a bx_message!!!
    
      // Set the message level (can be specified in the ctor even)
    void set_level (message_level level) { i4_message_level = level; }
      // Set the verbosity of this message: if it has to be printed and logged.
      // These methods are meant to be used by bx_base_module::get which knows
      // the module verbosity levels.
    void set_log_level (message_level level) { i4_log_level = level; }
    void set_print_level (message_level level) { i4_print_level = level; }
      // These methos are equivalent to the previus but accept a generic
      // vdt value (as got from the modules parameters map)
    void set_print_level (const vdt& level) { i4_print_level = m_decode(level); }
    void set_log_level (const vdt& level) { i4_log_level = m_decode(level); }

      // Get the string level representation
    const std::string& level () const { return message_codes[i4_message_level]; }
    
      // Some methods to chech the message verbosity for logs and printout
    bool check_log_level () const { return (i4_log_level == none) ? false : (i4_message_level >= i4_log_level); }
    bool check_print_level () const { return (i4_print_level == none) ? false : (i4_message_level >= i4_print_level); }
      // Chech if an exception has to be generated
    bool check_exception () const { return i4_message_level == critic; }
    
      // Dispatch the message to the messenger. Equivalent to message << dispatch
    void dispatch () { messenger::get ()->submit (*this); }
  private:
    message_level i4_print_level;
    message_level i4_log_level;
    message_level i4_message_level;
    typedef std::cmap<message_level, std::string> message_code_map;
    message_code_map message_codes;
    message_level m_decode (const vdt& v) const;
    static message_level default_print_level;
    static message_level default_log_level;
};

  // function to use bx_message << dispatch sintax. Equivalent to message.dispatch () 
inline std::ostream& dispatch (std::ostream& stream) {
  (dynamic_cast<bx_message&>(stream)).dispatch (); 
  return stream; 
}
#endif
/*
 * $Log: messenger.hh,v $
 * Revision 1.7  2004/04/12 16:07:40  razeto
 * Added configurable default print level for messages
 *
 * Revision 1.6  2004/04/09 07:47:28  razeto
 * Updated: changed the log level to info, added a string field to the bx_message ctor
 *
 * Revision 1.5  2004/04/06 12:39:28  razeto
 * Added logfile handling
 *
 * Revision 1.4  2004/04/05 13:10:23  razeto
 * Added comments, added an error level, changed some methods
 *
 * Revision 1.3  2004/04/03 09:21:53  razeto
 * fixed a bug
 *
 * Revision 1.2  2004/04/02 14:04:42  razeto
 * Added some comments. Fixed a typo
 *
 */
