/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 * 	   Marco Pallavicini <pallas@ge.infn.it>
 *
 * $Id: bx_base_module.hh,v 1.16 2004/04/24 17:36:45 razeto Exp $
 *
 * This is the pure base class for any module
 * A module is a piece of code performing any action on the data
 * See class bx_reco_framework for details
 * Each module has a pure virtual interface made of three methods:
 *    begin()    the operations the module should do BEFORE the event loop
 *    doit()     operation on each event
 *    end()      operations performed after the end of the event loop
 * Even if templates are possible for [sg]et_param_* they are not used to avoid
 * complicated code.
 *
 */
#ifndef _BX_BASE_MODULE_H
#define _BX_BASE_MODULE_H

#include "bx_reco_event.hh"
#include "bx_rec_general.hh"
#include "vdt.hh"
#include "messenger.hh"
#include "cmap.hh"

#include <string>
#include <vector>
#include <iostream>
#include <iomanip>


class bx_base_module {
  public:
    enum module_role {
      reader,
      precalib_cycle1,
      precalib_cycle2,
      precalib_cycle3,
      precalib_cycle4,
      main_loop,
      writer,
      all,
    };
    bx_base_module (const std::string& myname, module_role role);
    virtual ~bx_base_module () {}
      // Get the module name
    const std::string& get_name () const { return s_name; }
      // Get the module role
    module_role get_role () const { return i_role; }

      // Module parameter interface
    const vdt& get_parameter (const std::string& name) const { return param[name]; }
    bool check_parameter     (const std::string& name) const { return param.check (name); }
      // Init parameter create a parameter wit the given value
    void init_parameter      (const std::string& name, const vdt& value);
      // Set parameter set a parameter to a value. If the parameter was
      // not inited before a waning is generated.
    void set_parameter 	     (const std::string& name, const vdt& value);
    
      // some getters for the module status
    bool enabled () const { return b_enabled; }
    int get_priority () const { return i4_priority; } 
    
      // Operations for the framework 
    virtual void begin () = 0;
    virtual bx_reco_event* doit (bx_reco_event *ev) = 0;
    virtual void end () = 0;
    bx_reco_event * operator() (bx_reco_event *ev) { return doit (ev); }

    typedef std::vector<bx_base_module *> bx_base_module_vector;

      // Get a module message. 
      // !!! get_message give a initialized message, so if the message is used 
      // in multiline command a reference has been used
      // get_message () << line1;
      // get_message () << line2 << dispatch; only dumps line2 !!!!!!!
      // instead using a reference like bx_message &msg = get_message ();
      // msg << line1; msg << line2 << dispatch; is OK.
    bx_message& get_message (bx_message::message_level level = bx_message::none);

  private:
    std::string s_name;
    module_role i_role;
    bool b_enabled;
    long int i4_priority;
    typedef std::cmap<std::string, vdt> param_map;
    param_map param;
    bx_message message;
  
    bx_base_module (const bx_base_module& t) {} 
  public:
      // A list of module operation follow; a module operation can be used for
      // algorithms (see framework)
    struct begin_operation {
      begin_operation (module_role role): i_role(role) {}
      void operator() (bx_base_module *t) { if (t->enabled () && (i_role == t->get_role () || i_role == all)) t->begin (); }
      module_role i_role;
    };
    struct end_operation {
      end_operation (module_role role): i_role(role) {}
      void operator() (bx_base_module *t) { if (t->enabled () && (i_role == t->get_role () || i_role == all)) t->end (); }
      module_role i_role;
    };
    struct doit_operation {
      doit_operation (bx_reco_event *ev, module_role role): ev_saved(ev), i_role(role) {}
      void operator() (bx_base_module *t) { if (t->enabled () && (i_role == t->get_role () || i_role == all)) t->doit (ev_saved); }
      bx_reco_event *ev_saved;
      module_role i_role;
    };
    struct compare_name_operation {
      compare_name_operation (const std::string& module_name): s_name(module_name) {}
      bool operator() (const bx_base_module *t) { return s_name == t->get_name (); }
      std::string s_name;
    };
    struct compare_role_operation {
      compare_role_operation (bx_base_module::module_role role): i_role(role) {}
      bool operator() (const bx_base_module *t) { return t->enabled () ? i_role == t->get_role () : false; }
      bx_base_module::module_role i_role;
    };
    struct sort_operation {
      sort_operation () {}
      bool operator() (const bx_base_module *a, const bx_base_module *b) { 
        if (a->get_role () == b->get_role ()) return a->get_priority () <= b->get_priority ();
	else return a->get_role () < b->get_role ();
      }
    };
    struct delete_operation {
      delete_operation () {}
      void operator() (bx_base_module *t) { delete t; }
    };
};

inline std::ostream& operator<< (std::ostream &s, const bx_base_module* module) {
#if __GNUC__ >= 3
  s << "module: " << std::setw(30) << std::left << module->get_name ();
#else
  s << "module: " << std::setw(30) << module->get_name ();
#endif
  s << " role " << module->get_role ();
  s << " priority " << module->get_priority ();
  s << " enabled " << module->enabled () << std::endl;
  return s;
}

#endif
/*
 * $Log: bx_base_module.hh,v $
 * Revision 1.16  2004/04/24 17:36:45  razeto
 * Added a check if the parameter already exist in setting (added an init method too)
 *
 * Revision 1.15  2004/04/18 10:30:14  razeto
 * Fixed to compile wit g++ 2.95 present on the cluster
 *
 * Revision 1.14  2004/04/12 16:06:21  razeto
 * Fixed a bug. Changed syntax of begin and and operator
 *
 * Revision 1.13  2004/04/06 12:42:17  razeto
 * Added module role to base_module, and the code to handle everywhere. Some other minor fixes
 *
 * Revision 1.12  2004/04/05 13:07:45  razeto
 * Added messanger for exception handling. Updated a comment
 *
 * Revision 1.11  2004/04/02 14:04:42  razeto
 * Added some comments. Fixed a typo
 *
 * Revision 1.10  2004/04/02 13:39:36  razeto
 * Added messenger. Changed from map to cmap
 *
 * Revision 1.9  2004/04/01 12:08:53  razeto
 * Moved the base_module_operation operators in the bx_base_module class
 * scope; moved the bx_base_module_vector typedef in the bx_base_module
 * class.
 * Added a event delete to the frame.
 *
 * Revision 1.8  2004/03/29 13:15:40  razeto
 * Updated bx_base_module to the vtd usage
 *
 * Revision 1.7  2004/03/26 16:33:59  razeto
 * Added check_parameter to see if a parameter is defined without generating an exception
 *
 * Revision 1.6  2004/03/23 13:01:08  razeto
 * Moved base_module_operatos to bx_base_module.hh.
 * Added compare_operation.
 * Added get_module by name to framework.
 *
 * Revision 1.5  2004/03/22 17:26:10  razeto
 * Added module parameter interface
 *
 * Revision 1.4  2004/03/22 14:30:19  razeto
 * Moved bx_base_module_vector typedef in bx_base_module.hh
 *
 * Revision 1.3  2004/03/21 18:52:27  razeto
 * Removed "using std::string". Some cosmetic changes
 *
 * Revision 1.2  2004/03/20 17:39:45  pallas
 * First release
 *
 * Revision 1.1  2004/03/20 14:16:47  razeto
 * Added sources
 *
 */
