/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: bx_precalib_laben_gray.cc,v 1.4 2004/04/28 11:04:49 ddangelo Exp $
 *
 * Implementation of bx_precalibrator
 *
 */
#include "bx_precalib_laben_gray.hh"
#include "messenger.hh"
#include "laben_time_hit.hh"
#include "constants.hh"
#include "bx_dbi.hh"
#include "db_run.hh"

#include <algorithm>


// ctor
bx_precalib_laben_gray::bx_precalib_laben_gray (): bx_base_module("bx_precalib_laben_gray", bx_base_module::precalib_cycle1) {
}

// module interface
void bx_precalib_laben_gray::begin () {

  if (check_parameter ("channel_skip")) i_channel_skip = get_parameter ("channel_skip").get_int ();
  else i_channel_skip = 79;

  if (check_parameter ("histo_lenght")) i_histo_lenght = get_parameter ("histo_lenght").get_int ();
  else i_histo_lenght = 256;
  
  if (check_parameter ("mean_bound")) f_mean_bound = get_parameter ("mean_bound").get_float ();
  else f_mean_bound = 5;

  if (check_parameter ("rms_bound")) f_rms_bound = get_parameter ("rms_bound").get_float ();
  else f_rms_bound = 20;

    // allocate the matrix
  gray_map = new unsigned long[constants::laben.channels / i_channel_skip];
  gray_shift_map = new unsigned long**[constants::laben.channels / i_channel_skip];
  for (int j = 0; j < constants::laben.channels / i_channel_skip; j++) {
    gray_shift_map[j] = new unsigned long *[constants::laben.channels];
    for (int i = 0; i < constants::laben.channels; i++)
      gray_shift_map[j][i] = new unsigned long[i_histo_lenght](0);
  }

  get_message(bx_message::debug) << "begin" << dispatch;
}

bx_reco_event* bx_precalib_laben_gray::doit (bx_reco_event *ev) {
  const bx_laben_event& er = ev->get_laben();
    // First set the refernce channels
  for (int i = 0; i < er.get_raw_nhits (); i++) {
    const bx_laben_raw_hit &hit = er.get_raw_hit (i);
    for (int j = 0; j < constants::laben.channels / i_channel_skip; j++) 
      if (hit.get_channel () == i_channel_skip * (j + 1) && hit.get_time_1 () != 0xff && hit.get_time_2 () != 0xff)
        gray_map[j] = hit.get_gray_counter ();
  }
    // Then fill the shift histogram
  for (int i = 0; i < er.get_raw_nhits (); i++) {
    const bx_laben_raw_hit &hit = er.get_raw_hit (i);
    if (hit.get_time_1 () == 0xff && hit.get_time_2 () == 0xff) continue;
    unsigned long gray = hit.get_gray_counter ();
    for (int j = 0; j < constants::laben.channels / i_channel_skip; j++) {
      int bin = gray - gray_map[j] + i_histo_lenght / 2;
      if (bin < 0) bin = 0;
      else if (bin >= i_histo_lenght) bin = i_histo_lenght - 1;
      gray_shift_map[j][hit.get_channel ()][bin]++;
    }
  }
  return ev;
}

void bx_precalib_laben_gray::m_statistic_info (unsigned long *v, int& max_bin) {
  unsigned long max_value = 0;

  for (int i = 0; i < i_histo_lenght; i++) {
    if (v[i] > max_value) {
      max_bin = i;
      max_value = v[i];
    }
  }
  max_bin -= i_histo_lenght / 2;
}

bool bx_precalib_laben_gray::m_check_bound (unsigned long **v, float& mean, float& rms) {
  unsigned long sum = 0, hits = 0;
  
  for (int j = 0; j < constants::laben.channels; j++) {
    for (int i = 0; i < i_histo_lenght; i++) {
      sum += i * v[j][i];
      hits += v[j][i];
    }
  }
  mean = (double)sum / hits;

  double l2 = 0;
  for (int j = 0; j < constants::laben.channels; j++) {
    for (int i = 0; i < i_histo_lenght; i++) {
      l2 += (mean - i) * (mean - i) * v[j][i];
    }
  }
  rms = ::sqrt (l2) / hits;
  
  mean -= i_histo_lenght / 2;
  return mean < f_mean_bound && rms < f_rms_bound;
}
  
void bx_precalib_laben_gray::end () {

  db_run& run_info = bx_dbi::get ()->get_run_nonconst (bx_dbi::get ()->get_current_run_number ());

  bool found = false;

  for (int j = 0; j < constants::laben.channels / i_channel_skip; j++) {
    float mean, rms;
    if (m_check_bound (gray_shift_map[j], mean, rms)) {
      get_message (bx_message::info) << "found channel reference " << i_channel_skip * (j + 1) << " with shift " << mean << "+-" << rms << dispatch;
      for (int i = 0; i < constants::laben.channels; i++) {
	int max_bin;
	m_statistic_info (gray_shift_map[j][i], max_bin);
	run_info.set_laben_precalib_gray_shift (i, max_bin, this);
      }
      found = true;
      break;
    }
  }
  if (!found) get_message (bx_message::critic) << "no gray shift reference channel found" << dispatch;    


    // delete the matrix
  delete [] gray_map;
  for (int j = 0; j < constants::laben.channels / i_channel_skip; j++) {
    for (int i = 0; i < constants::laben.channels; i++)
      delete [] gray_shift_map[j][i];
    delete [] gray_shift_map[j];
  }
  get_message(bx_message::debug) << "end" << dispatch;
}
/*
 * $Log: bx_precalib_laben_gray.cc,v $
 * Revision 1.4  2004/04/28 11:04:49  ddangelo
 * Modules adapted to call the right const/nonconst event portion getter
 *
 * Revision 1.3  2004/04/27 14:25:18  ddangelo
 * Modifications to match new event structure.
 * Useless dynamic_cast in event reading replaced by getter.
 * safer const reference is used instead of non-const pointer.
 *
 * Revision 1.2  2004/04/26 13:53:00  razeto
 * Updated to follow new name/syntax of db_run and db_profile
 *
 * Revision 1.1  2004/04/12 15:59:49  razeto
 * Added a firsto core of laben precalibration
 *
 */
