/* BOREXINO Reconstruction program
 *
 * Author: Davide D'Angelo <davide.dangelo@lngs.infn.it>
 *
 * $Id: bx_precalib_muon_pedestals.cc,v 1.5 2004/04/28 11:04:49 ddangelo Exp $
 *
 * Implementation of bx_precalib_muon_pedestals.hh
 *
 */

#include <stdlib.h>
#include <math.h>

#include "bx_precalib_muon_pedestals.hh"
#include "messenger.hh"
#include "constants.hh"
#include "bx_dbi.hh"
#include "db_run.hh"

const unsigned short bx_precalib_muon_pedestals::min_pedestal_width = 620; 
const unsigned short bx_precalib_muon_pedestals::max_pedestal_width = 810; 
const unsigned short bx_precalib_muon_pedestals::precalib_pulse_tolerance = 128;  /* Allowed time jitter on precalib pulse (TDC ticks) */

// ctor
bx_precalib_muon_pedestals::bx_precalib_muon_pedestals (): bx_base_module("bx_precalib_muon_pedestals", bx_base_module::precalib_cycle2) {
}

void bx_precalib_muon_pedestals::begin () {
  for (int i = 0; i < constants::muon.channels; i++) pedestals[i] = new std::vector <unsigned long>;

  get_message(bx_message::debug) << "begin" << dispatch;
}

bx_reco_event* bx_precalib_muon_pedestals::doit (bx_reco_event *ev) {
  const bx_muon_event& er = ev->get_muon();

  const db_run& run_info = bx_dbi::get ()->get_run (bx_dbi::get ()->get_current_run_number ());
  unsigned long pulse_time = run_info.get_muon_precalib_pulse_time();
 
  for (unsigned long i = 0; i <er.get_raw_nhits(); i++) { 
    const bx_muon_raw_hit& h = er.get_raw_hit(i);

    // skip special channels
    if (!h.is_pmt_channel())
      continue;

    // skip hits unrelated to primiry led emission (time filter)
    if (::abs(h.get_lead_time() - pulse_time) > precalib_pulse_tolerance )
      continue;

    // check if width is ok
    if ( h.get_time_diff() >= min_pedestal_width && 
	 h.get_time_diff() <= max_pedestal_width)
      pedestals[h.get_muon_channel()]->push_back (h.get_time_diff());
    else
      get_message(bx_message::warn) << "Unusual pedestal width (skipping); ev " << ev->get_event_number() << " mch " << h.get_muon_channel() << " width " << h.get_time_diff() << dispatch; 
  } // end ov loop over hits
  return ev;
}

void bx_precalib_muon_pedestals::end () {
  db_run& run_info = bx_dbi::get ()->get_run_nonconst (bx_dbi::get ()->get_current_run_number ());

  for(int i=0; i<constants::muon.channels; i++) { 
    unsigned long sum = 0;
    for (unsigned int j=0; j < pedestals[i]->size(); j++)
      sum += (*pedestals[i])[j];
    double mean = (pedestals[i]->size() > 0) ? double(sum) / pedestals[i]->size() : 0.;
    double quad_sum = 0.;
    for (unsigned int j=0; j < pedestals[i]->size(); j++)
      quad_sum += ::pow((*pedestals[i])[j] - mean, 2);
    double rms = (pedestals[i]->size() > 0) ? ::sqrt(quad_sum / pedestals[i]->size()) : 0.;
    run_info.set_muon_precalib_pedestal(i, mean, this);	
    get_message(bx_message::info) << "mch " << i << " entries " << pedestals[i]->size() << " ped " << mean << " rms " << rms << dispatch;
    delete pedestals[i];
  } /* end of loop on channels */
  get_message(bx_message::debug) << "end" << dispatch;
}  

/*
 * $Log: bx_precalib_muon_pedestals.cc,v $
 * Revision 1.5  2004/04/28 11:04:49  ddangelo
 * Modules adapted to call the right const/nonconst event portion getter
 *
 * Revision 1.4  2004/04/28 10:48:21  ddangelo
 * Pedestal calculation reworked.
 *
 * Revision 1.3  2004/04/27 09:46:50  ddangelo
 * modifications to match new event structure.
 * Other debugging
 *
 * Revision 1.1  2004/04/20 11:36:25  ddangelo
 * added 2 modules for outer muon precalibration
 *
 *
 */
